./ ADD NAME=REGEXDEM
      ****************************************************************
      * This file is a demonstration program of using the IBM, POXIX *
      * compliant, runtime library implementation of Regular         *
      * Expressions pattern matching functions in COBOL.  I would try*
      * to demonstrate the usage of the various copybooks that where *
      * created from breaking the regex.h and how to capture a       *
      * matched substring in the match structure.                    *
      * Adaptation of a COBOL program was written by Frank Swarbrick *
      * Note: leading underscores in a name are marked with U        *
      *       other underscores converted to dashes                  *
      ****************************************************************
      *
      * Copyright (c) 2015 Ze'ev Atlas
      * Please refer to the LICENSE document to see all other
      * applicable copyrights.
      *
      *---------------------------------------------------------------
      *Redistribution and use in source and binary forms, with or
      *without modification, are permitted provided that the following
      *conditions are met:

      * 1. Redistributions of source code must retain the above
      * copyright notice, this list of conditions and the following
      * disclaimer.

      * 2. Redistributions in binary form must reproduce the above
      * copyright notice, this list of conditions and the following
      * disclaimer in the documentation and/or other materials
      * provided with the distribution.

      * 3. Neither the name of Ze'ev Atlas nor the names of other
      * contributors may be used to endorse or promote
      * products derived from this software without specific prior
      * written permission.

      *THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
      *CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
      *INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
      *MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
      *DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
      *CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
      *SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
      *NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
      *LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
      *HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
      *CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
      *OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
      *EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
      * Note about this implementation
      * The original IBM go to a long distance to create either
      * separate regext and regmatch structures, one for ASCII and one
      * for EBCDIC, or default to one of them.  I went only for the
      * separate option and created two versions for completenesee.
      * Since this implementation is geared towards EBCDIC, one should
      * probably always use the EBCDIC version (file and structure
      * name end with 'e'.  There is a default value that is not used
      * anyway but is defaulted to the EBCDIC anyway.
      ****************************************************************
      ****process nodynam pgmname(mixed)
      *    process codepage(1047)
       identification division.
       program-id.  'REGEXDEM'.
       data division.
       working-storage section.
      * This brings in all constants.
        copy regex.
      * RE1-regex-t-e is the regex_t structure.  Using the replacing
      * clause allow for multiple instances.
        copy regext replacing ==:PREFIX:== BY ==RE1==.
      * The below construct is puzzling for me.  Theoretically
      * regmatc-t-e should have worked, but the result is clearly
      * regmatch-t-a.  Moreover, it looks like the first element
      * of the first element of that construct behaves as if it was
      * int (32 bits) while the rest of the elements behave as if
      * they were long (64 bits).  I created the convoluted construct
      * below to overcome this issue, but I certainly need to
      * investigate it!
       01 RM1-match.
      * RM1-regmatch-t is the regmatch_t structure.  Using the
      * replacing clause allow for multiple instances.
          05 FILLER OCCURS 4.
        copy regmtch replacing ==:PREFIX:== BY ==RM1==.

       01  msgbuf              display   pic x(100).

       01 my-string            PIC X(255).
       01 my-ptr               usage pointer.
       01 my-len               PIC s9(9) comp-5.

       procedure division.
           string 'a simple string' delimited by size
                  x'00' delimited by size
                  into my-string
           end-string
           move LENGTH OF RE1-regex-t to my-len
           call 'regcomp' using RE1-regex-t
                                content z'.*(simple).*'
                                value REG-EXTENDED
                returning REG-ERROR-CODE
      * extern void DUMPMEM(char *address, int length);
      *    call 'DUMPMEM' using BY REFERENCE RE1-regex-t
                                value my-len
           display my-len
           if REG-ERROR-CODE is not equal to zero
               display 'Could not compile regex'
               display REG-ERROR-CODE
               stop run
           end-if
           call 'regexec' using RE1-regex-t
                                my-string
                                value 2
                                by reference RM1-match
                                value 0
                returning REG-ERROR-CODE
      *    call 'DUMPMEM' using RM1-match value 32
           perform check-reti

           call 'regfree' using RE1-regex-t
           goback.

       check-reti.
           evaluate REG-ERROR-CODE
           when zero
               display 'match'
               display RM1-rm-so(1)
      *        display RM1-rm-ss(1)
               display RM1-rm-eo(1)
      *        display RM1-rm-es(1)
               display my-string (RM1-rm-so(1) + 1:
                       RM1-rm-eo(1) - RM1-rm-so(1))
               display RM1-rm-so(2)
      *        display RM1-rm-ss(2)
               display RM1-rm-eo(2)
      *        display RM1-rm-es(2)
               display my-string (RM1-rm-so(2) + 1:
                       RM1-rm-eo(2) - RM1-rm-so(2))
      *        display RM1-rm-so(3)
      *        display RM1-rm-eo(3)
      *        display RM1-rm-so(4)
      *        display RM1-rm-eo(4)
           when 1
               display 'no match'
           when other
      *        move length of msgbuf to lmsgbuf
               call 'regerror' using RE1-regex-t
                                     msgbuf
                                     value length of msgbuf
                    returning REG-ERROR-CODE
               display 'Regex match failed: '
               display REG-ERROR-CODE
               display msgbuf
               stop run
           end-evaluate
           .

       end program 'REGEXDEM'.

